@@ -8,9 +8,11 @@ program test
     integer, pointer :: p
   end type
   type(with_pointer) :: coindexed[*]
-  integer, pointer :: scalarIntF, arrayIntF(:)
+  integer, pointer :: scalarIntF, arrayIntF(:), multiDimIntF(:,:)
   character(len=:), pointer :: charDeferredF
   integer :: j
+  integer, dimension(2, 2) :: rankTwoArray
+  rankTwoArray = reshape([1, 2, 3, 4], shape(rankTwoArray))
   call c_f_pointer(scalarC, scalarIntF) ! ok
   call c_f_pointer(scalarC, arrayIntF, [1_8]) ! ok
   call c_f_pointer(shape=[1_8], cptr=scalarC, fptr=arrayIntF) ! ok
@@ -19,7 +21,7 @@ program test
   call c_f_pointer(scalarC, fptr=arrayIntF, [1_8])
   !ERROR: CPTR= argument to C_F_POINTER() must be a C_PTR
   call c_f_pointer(j, scalarIntF)
-  !ERROR: CPTR= argument to C_F_POINTER() must be scalar
+  !ERROR: Rank of dummy argument is 0, but actual argument has rank 1
   call c_f_pointer(arrayC, scalarIntF)
   !ERROR: SHAPE= argument to C_F_POINTER() must appear when FPTR= is an array
   call c_f_pointer(scalarC, arrayIntF)
@@ -29,4 +31,10 @@ program test
   call c_f_pointer(scalarC, charDeferredF)
   !ERROR: FPTR= argument to C_F_POINTER() may not be a coindexed object
   call c_f_pointer(scalarC, coindexed[0]%p)
+  !ERROR: FPTR= argument to C_F_POINTER() must have a type
+  call c_f_pointer(scalarC, null())
+  !ERROR: SHAPE= argument to C_F_POINTER() must have size equal to the rank of FPTR=
+  call c_f_pointer(scalarC, multiDimIntF, shape=[1_8])
+  !ERROR: SHAPE= argument to C_F_POINTER() must be a rank-one array.
+  call c_f_pointer(scalarC, multiDimIntF, shape=rankTwoArray)
 end program
