@@ -13,10 +13,10 @@
 // UNSUPPORTED: gcc
 
 // Clang does not support the -fexperimental-library flag before LLVM 15.0
-// UNSUPPORTED: clang-13, clang-14
+// UNSUPPORTED: clang-14
 
 // AppleClang does not support the -fexperimental-library flag yet
-// UNSUPPORTED: apple-clang-13, apple-clang-14.0
+// UNSUPPORTED: apple-clang-14.0
 
 // Clang on AIX currently pretends that it is Clang 15, even though it is not (as of writing
 // this, LLVM 15 hasn't even been branched yet).
@@ -29,7 +29,3 @@
 #ifdef _LIBCPP_HAS_NO_INCOMPLETE_FORMAT
 #   error "-fexperimental-library should enable <format>"
 #endif
-
-#ifdef _LIBCPP_HAS_NO_INCOMPLETE_RANGES
-#   error "-fexperimental-library should enable <ranges>"
-#endif
