@@ -6,6 +6,8 @@
 //
 //===----------------------------------------------------------------------===//
 
+// UNSUPPORTED: c++03, c++11
+
 // <map>
 
 // class multimap
@@ -24,10 +26,6 @@
 #include "test_macros.h"
 #include "is_transparent.h"
 
-#if TEST_STD_VER <= 11
-#error "This test requires is C++14 (or later)"
-#else
-
 int main(int, char**)
 {
     {
@@ -36,4 +34,3 @@ int main(int, char**)
     TEST_IGNORE_NODISCARD M().find(C2Int{5});
     }
 }
-#endif
